/*
 *
 * HGA v1.3.1
 * Program: makepop, which generates input files for the HGA
 * File: makepop.c, Contains main code of program
 *
 * Copyright (c) 1997, 1995, 1994 Stephen D. Scott and the University of
 * Nebraska-Lincoln.  All rights reserved.
 *
 * Permission to use or modify this software and its documentation only
 * for educational purposes and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies.  The author
 * and university make no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.  The author and university shall not be liable in
 * any event for incidental or consequential damages in connection
 * with, or arising out of, the furnishing, performance, or use of
 * these programs.
 *
 */


#define MAIN
#include "makepop.h"

main(int argc, char *argv[])
{
char *line, err[50];
int i, fitness, tmp; 
unsigned long long sof=0, newmemb, limit, val;
FILE *tmpscript;

Init();

CheckArgs(argc, argv);

if (n+f > valw)
  Error("n+f > valw");

if (p > valw)
  Error("p > valw");

if (out == NULL)
  {
    fprintf(stderr, "Error: specify output file\n");
    Usage(NULL, argv[0]);
  }

if (--m >= (limit = mypow(2, valw)))
  Error("m is too large for current value of valw");
if (--g >= limit)
  Error("g is too large for current value of valw");

if (2*m + 8 > memsize)
  Error("2*m+6 > memsize");

if ((tmpscript = fopen(".tmpscript.csh", "w")) == NULL)
  Error("Could not open file .tmpscript.csh");

if ((line = (char *) malloc((valw + 1) * sizeof(char))) == NULL)
  Error("Problem allocating line");

line[valw] = '\0';

/* seed random number generator */
initrnd();

/* psize */
if (ToBinary((unsigned long long) m, line, valw) <= 0)
  Error("Problem writing pop. size");
else
  fprintf(out, "%s\n", line);

/* rng seed */
for (i=0; i<valw; i++)
  line[i] = flip(0.5);
fprintf(out, "%s\n", line);

/* mut. prob. */
if (ToBinary((unsigned long long) (mutprob * pow(2., (double) p)), line, valw) <= 0)
  Error("Problem writing mut. prob.");
else
  fprintf(out, "%s\n", line);

/* xov. prob. */
if (ToBinary((unsigned long long) (xovprob * pow(2., (double) p)), line, valw) <= 0)
  Error("Problem writing xov. prob.");
else
  fprintf(out, "%s\n", line);

/* init. s.o.f. */
for (i=0; i<valw; i++)
  line[i] = 'X';
fprintf(out, "%s\n", line);
fprintf(tmpscript, "#!/bin/csh\n\n");
fprintf(tmpscript, "mv %s %s.spamspam\n", fname, fname);
fprintf(tmpscript, "sed \'s/%s/", line);

/* no. of gens. */
if (ToBinary((unsigned long long) g, line, valw) <= 0)
  Error("Problem writing num. gens.");
else
  fprintf(out, "%s\n", line);

limit = mypow(2, n) - 1;
for (i=0; i<=m; i++)
  {
   /* generate memb. */
    newmemb = rnd(0,limit);
   /* evaluate and accum. sof */
    sof += fitness = Eval(newmemb);
   /* write into string:
    * the low f bits get fitness and the next higher n bits get newmemb */
    if ((tmp = ToBinary(val = (((unsigned long long) newmemb * 
       mypow(2, f)) + fitness), line, valw)) <= 0)
      {
        if (tmp == -1) 
          {
            sprintf(err, "When writing pop. member, its value %lu < 0", val);
            Error(err);
          }
        if (tmp == -2)
          {
            sprintf(err, "When writing pop. member, its value %lu >= max size %lu", val, mypow(2, valw));
            Error(err);
          }
        else
          Error("When writing pop. member, unknown error occurred");
      }

    else
      fprintf(out, "%s\n", line);
  }

/* make population #2 all 0s */
for (i=0; i<valw; i++)
  line[i] = '0';
for (i=0; i<=m; i++)
  fprintf(out, "%s\n", line);

/* pad remaining slots with 1s */
for (i=0; i<valw; i++)
  line[i] = '1';
for (i=2*m+8; i<memsize; i++)
  fprintf(out, "%s\n", line);

if (ToBinary(sof, line, valw) == 0)
  Error("Problem writing sof");
fprintf(tmpscript, "%s/\' < %s.spamspam > %s\n", line, fname, fname);
fprintf(tmpscript, "\\rm -f %s.spamspam .tmpscript.csh\n", fname);
fclose(out);
fclose(tmpscript);
system("chmod 700 .tmpscript.csh");
execlp(".tmpscript.csh", NULL);

}

int ToBinary(unsigned long long val, char *string, int size)
{
int i;

if (val < 0) 
  return -1;
if (val >= mypow(2, size))
  return -2;

for (i=size-1; i>=0; i--)
  {
    if ((val % 2) == 1)
      {
        string[i] = '1';
        val--;
      }
    else
      string[i] = '0';
    val /= 2;
  }

return 1;
}


unsigned long long mypow(base, exp)
{
unsigned long long tmp=1;
int i;

for (i=0;i<exp;i++)
  tmp *= base;
return tmp;
}
